// Generated by CoffeeScript 1.7.1
(function() {
  var scope, _ref, _ref1;

  scope = typeof window !== "undefined" && window !== null ? window : this;

  scope.NumSync = (_ref = scope.NumSync) != null ? _ref : {};

  scope.NumSync.EntryHelper = (_ref1 = scope.NumSync.EntryHelper) != null ? _ref1 : {};

  scope.NumSync.EntryHelper.contract = {
    openEntry: function(entry) {},
    shareEntry: function(entry) {},
    removeEntries: function(entries) {},
    moveEntries: function(entries) {},
    renameEntry: function(entry) {}
  };

  scope.NumSync.EntryHelper.impl = {
    hFM: null,
    i18n: null,
    getComponentName: function() {
      return 'EntryHelper';
    },
    configure: function(hub, configuration) {
      this.hub = hub;
      this.hub.requireService({
        component: this,
        contract: scope.NumSync.FileManager.contract,
        field: "hFM"
      });
      this.hub.requireService({
        component: this,
        contract: scope.Ubidreams.i18n.contract,
        field: "i18n"
      });
      return this.hub.provideService({
        component: this,
        contract: scope.NumSync.EntryHelper.contract
      });
    },
    start: function() {},
    stop: function() {},

    /*
      * Open an entry via DocumentInteraction plugin with view mode
      * @param <json> entry
      * @param <json> opts : opts with callbacks
     */
    openEntry: function(entry, opts) {
      return this._performOpen(entry, false, opts);
    },

    /*
      * Share an entry via DocumentInteraction plugin with share mode
      * (Only for Android)
      * @param <json> entry
     */
    shareEntry: function(entry) {
      return this._performOpen(entry, true);
    },

    /*
      * Rename an entry
     */
    renameEntry: function(entry) {
      var onValidate;
      onValidate = Uju.bind(this, function(renameModal, name) {
        var action, defaultParams, onComplete, path, targetPayload;
        defaultParams = true;
        onComplete = {
          onSuccess: Uju.bind(this, function() {
            return renameModal.renameSuccess();
          }),
          onError: Uju.bind(this, function(message, status) {
            return renameModal.renameFailed(message);
          })
        };
        switch (entry.type) {
          case NumSync.models.Entry.types.CLOUD_FOLDER:
            action = "updateCloudFolder";
            defaultParams = false;
            break;
          case NumSync.models.Entry.types.FOLDER:
            action = "updateFolder";
            break;
          case NumSync.models.Entry.types.FILE:
            action = "updateFile";
        }
        if (defaultParams) {
          targetPayload = entry.getPayload();
          path = targetPayload.path.split("/");
          path.pop();
          path = path.join("/");
          targetPayload.path = (path ? path + "/" : "") + name;
          return this.hFM[action](entry.idCloudFolder, entry.path, targetPayload, onComplete);
        } else {
          targetPayload = entry.getPayload();
          targetPayload.name = name;
          return this.hFM[action](targetPayload, onComplete);
        }
      });
      return enyo.Signals.send("showModal", {
        clean: false,
        modal: {
          className: "NumSync.RenameEntry",
          config: {
            onValidate: onValidate,
            entryName: entry.getEntryName()
          }
        }
      });
    },

    /*
      * Remove given entries
      * @param <array> entries
     */
    removeEntries: function(entries) {
      return navigator.notification.confirm(this.i18n.getTranslation("remove-msg"), Uju.bind(this, function(buttonIndex) {
        if (buttonIndex === 2) {
          return this._performRemove(entries);
        }
      }), this.i18n.getTranslation("remove-title", [entries.length]), [this.i18n.getTranslation("cancel"), this.i18n.getTranslation("remove")]);
    },

    /*
      * Move given entry
      * @param <array> entries
     */
    moveEntries: function(entries) {
      return enyo.Signals.send("requestFilesTreeLight", {
        widgetConf: {
          onDropFile: Uju.bind(this, this._performMove, entries)
        }
      });
    },

    /*
      * Open an entry calling native dialog
      * @param <json> entry
      * @param <boolean> shareOnly : see DocumentInteraction plugin => no preview, call share dialog (Android only)
      * @param <json> opts = {
      *    onOpen : <function> triggered when open is completed with success or error
      *    notExist : <function> triggered when a file does not exists from server
      * }
     */
    _performOpen: function(entry, shareOnly, opts) {
      var bindAbort;
      bindAbort = false;
      return this.hFM.getFile(entry.idCloudFolder, entry.path, {
        onProgress: Uju.bind(this, function(event, abort, modal) {
          var percentage;
          if (bindAbort === false) {
            modal._onClose = function() {
              abort();
              return opts != null ? typeof opts.onOpen === "function" ? opts.onOpen() : void 0 : void 0;
            };
            modal.canAbort();
            bindAbort = true;
          }
          if (event.lengthComputable) {
            percentage = event.loaded / event.total * 100;
            return modal.setPercentage(parseInt(percentage));
          }
        }),
        onSuccess: Uju.bind(this, function(file, modal) {
          if (opts != null) {
            if (typeof opts.onOpen === "function") {
              opts.onOpen();
            }
          }
          if (modal != null) {
            modal.endDownload();
            return modal._onClose = Uju.bind(this, function() {
              return this._callDocumentInteractionPlugin(file.fullPath, shareOnly);
            });
          } else {
            return this._callDocumentInteractionPlugin(file.fullPath, shareOnly);
          }
        }),
        onDownload: Uju.bind(this, function(start) {
          return enyo.Signals.send("showModal", {
            clean: (shareOnly === true ? false : true),
            modal: {
              kind: "NumSync.modals.Downloading",
              entry: entry,
              handlers: {
                onOpen: "_onOpen",
                onClose: "_onClose"
              },
              _onOpen: function() {
                return start(this);
              }
            }
          });
        }),
        onError: Uju.bind(this, function(message, status) {
          if (status === NumSync.RESTapi.error.RESOURCE_NOT_FOUND) {
            if (opts != null) {
              if (typeof opts.notExist === "function") {
                opts.notExist();
              }
            }
          }
          if (opts != null) {
            if (typeof opts.onOpen === "function") {
              opts.onOpen();
            }
          }
          return this._onActionError(message);
        })
      });
    },

    /*
      * Call Document Interaction Phonegap plugin
      * @param <string> filePath
      * @param <boolean> shareOnly : see DocumentInteraction plugin => no preview, call share dialog (Android only)
     */
    _callDocumentInteractionPlugin: function(filePath, shareOnly) {
      return DocumentInteraction.openDocument(filePath, null, shareOnly, {
        onError: Uju.bind(this, function() {
          if (shareOnly) {
            return this._onActionError(this.i18n.getTranslation("export_error"));
          } else {
            return this._onActionError(this.i18n.getTranslation("open_error"));
          }
        })
      });
    },

    /*
      * Remove an entry
      * @param <array> entries
     */
    _performRemove: function(entries) {
      var entry, i, onComplete, tasks, _i, _len;
      onComplete = Uju.bind(this, function(success_entries, error_messages) {
        var errorMessage, successMessage;
        successMessage = this.i18n.getTranslation("remove-success", [success_entries.length]);
        errorMessage = this.i18n.getTranslation("remove-error", [error_messages.length, error_messages[0]]);
        return this._onActionComplete(success_entries, error_messages, successMessage, errorMessage);
      });
      this._onActionProgress(this.i18n.getTranslation("remove-pending"));
      tasks = {};
      for (i = _i = 0, _len = entries.length; _i < _len; i = ++_i) {
        entry = entries[i];
        tasks[i] = Uju.bind(this, function(_i, _entry, callback) {
          var onError, onSuccess;
          onSuccess = Uju.bind(this, function(data, header) {
            return callback(null, "SUCCESS", header);
          });
          onError = Uju.bind(this, function(message, status) {
            return callback(null, "ERROR", message);
          });
          switch (_entry.type) {
            case NumSync.models.Entry.types.CLOUD_FOLDER:
              return this.hFM.deleteCloudFolder(_entry.id, {
                onSuccess: onSuccess,
                onError: onError
              });
            case NumSync.models.Entry.types.FOLDER:
              return this.hFM.deleteFolder(_entry.idCloudFolder, _entry.path, {
                onSuccess: onSuccess,
                onError: onError
              });
            case NumSync.models.Entry.types.FILE:
              return this.hFM.deleteFile(_entry.idCloudFolder, _entry.path, {
                onSuccess: onSuccess,
                onError: onError
              });
          }
        }, i, entry);
      }
      return async.parallel(tasks, Uju.bind(this, function(err, results) {
        var e, key, result, s;
        e = [];
        s = [];
        for (key in results) {
          result = results[key];
          if (result[0] === "SUCCESS") {
            s.push(result[1]);
          } else {
            e.push(result[1]);
          }
        }
        return onComplete(s, e);
      }));
    },

    /*
      * Move an entry
      * @param <array> entries
      * @param <json> targetEntry : the current folder entry where the entry will be moved
      * @param <function> close : to close the files tree light
     */
    _performMove: function(entries, targetEntry, close) {
      var entry, i, onComplete, targetPayload, tasks, _i, _len;
      onComplete = Uju.bind(this, function(success_entries, error_messages) {
        var errorMessage, parent, path, successMessage;
        close();
        path = targetEntry.path || "";
        path += targetEntry.path ? "/" : "";
        parent = targetEntry.id != null ? targetEntry.name : targetEntry._parent_;
        successMessage = this.i18n.getTranslation("move-success", [success_entries.length, parent + "/" + path]);
        errorMessage = this.i18n.getTranslation("move-error", [error_messages.length, error_messages[0]]);
        return this._onActionComplete(success_entries, error_messages, successMessage, errorMessage);
      });
      this._onActionProgress(this.i18n.getTranslation("move-pending"));
      tasks = {};
      for (i = _i = 0, _len = entries.length; _i < _len; i = ++_i) {
        entry = entries[i];
        targetPayload = {};
        targetPayload.idCloudFolder = targetEntry.id || targetEntry.idCloudFolder;
        targetPayload.path = targetEntry.path || "";
        if (targetPayload.path !== "") {
          targetPayload.path += "/";
        }
        targetPayload.path += entry.getEntryName();
        tasks[i] = Uju.bind(this, function(_i, _entry, _targetPayload, callback) {
          var action;
          switch (_entry.type) {
            case NumSync.models.Entry.types.FOLDER:
              action = "updateFolder";
              break;
            case NumSync.models.Entry.types.FILE:
              action = "updateFile";
          }
          return this.hFM[action](_entry.idCloudFolder, _entry.path, _targetPayload, {
            onSuccess: Uju.bind(this, function(data, header) {
              return callback(null, "SUCCESS", header);
            }),
            onError: Uju.bind(this, function(message, status) {
              return callback(null, "ERROR", message);
            })
          });
        }, i, entry, targetPayload);
      }
      return async.parallel(tasks, Uju.bind(this, function(err, results) {
        var e, key, result, s;
        e = [];
        s = [];
        for (key in results) {
          result = results[key];
          if (result[0] === "SUCCESS") {
            s.push(result[1]);
          } else {
            e.push(result[1]);
          }
        }
        return onComplete(s, e);
      }));
    },

    /*
      * Progress callback for "remove", "move" actions
      * @param <string> message
     */
    _onActionProgress: function(message) {
      this._startProgress = new Date().getTime();
      return enyo.Signals.send("showModal", {
        clean: false,
        modal: {
          kind: "NumSync.Notification",
          type: NumSync.Notification.types.PENDING,
          message: message
        }
      });
    },

    /*
      * Success callback for "remove", "move" actions
      * @param <string> message
      * @param <function> onClose
     */
    _onActionSuccess: function(message, onClose) {
      var waiting;
      waiting = 1000 - (new Date().getTime() - this._startProgress);
      if (waiting < 0) {
        waiting = 0;
      }
      return setTimeout(Uju.bind(this, function() {
        enyo.Signals.send("showModal", {
          clean: true,
          modal: {
            kind: "NumSync.Notification",
            type: NumSync.Notification.types.SUCCESS,
            message: message,
            timeout: 4000,
            onClose: onClose
          }
        });
        return enyo.Signals.send("refreshFilesTree", {});
      }), waiting);
    },

    /*
      * Error callback for "remove", "move" actions
      * @param <string> message
     */
    _onActionError: function(message) {
      var waiting;
      waiting = 1000 - (new Date().getTime() - this._startProgress);
      if (waiting < 0) {
        waiting = 0;
      }
      return setTimeout(Uju.bind(this, function() {
        return enyo.Signals.send("showModal", {
          clean: true,
          modal: {
            kind: "NumSync.Notification",
            type: NumSync.Notification.types.ERROR,
            message: message,
            timeout: 4000
          }
        });
      }), waiting);
    },

    /*
      * Complete callback for "remove", "move" actions
      * @param <string> message
     */
    _onActionComplete: function(success_entries, error_messages, successMessage, errorMessage) {
      enyo.Signals.send("requestSelections", {
        selecting: false
      });
      enyo.Signals.send("toggleFooter", {
        show: false
      });
      if (success_entries.length > 0) {
        return this._onActionSuccess(successMessage, Uju.bind(this, function() {
          if (error_messages.length > 0) {
            return this._onActionError(errorMessage);
          }
        }));
      } else {
        return this._onActionError(errorMessage);
      }
    }
  };

}).call(this);
