// Generated by CoffeeScript 1.7.1
(function() {
  var scope, _ref, _ref1;

  scope = typeof window !== "undefined" && window !== null ? window : this;

  scope.NumSync = (_ref = scope.NumSync) != null ? _ref : {};

  scope.NumSync.Upload = (_ref1 = scope.NumSync.Upload) != null ? _ref1 : {};

  scope.NumSync.Upload.contract = {
    getQueueForView: function() {},
    getUploadingForView: function() {},
    setProgressTracker: function(callback) {}
  };

  scope.NumSync.Upload.impl = {
    _queue: {
      working: false,
      items: [],
      failures: []
    },
    uploading: null,
    progressTracker: null,
    getComponentName: function() {
      return "Upload";
    },
    configure: function(hub, configuration) {
      this.hub = hub;
      this.hub.requireService({
        component: this,
        contract: scope.Ubidreams.i18n.contract,
        field: "i18n"
      });
      this.hub.requireService({
        component: this,
        contract: scope.NumSync.FileManager.contract,
        field: "hFM"
      });
      this.hub.requireService({
        component: this,
        contract: scope.Ubidreams.FileSystem.contract,
        field: "hFS"
      });
      this.hub.requireService({
        component: this,
        contract: scope.Ubidreams.ConnectivityWatcher.contract,
        field: "hCW"
      });
      this.hub.provideService({
        component: this,
        contract: scope.NumSync.Upload.contract
      });
      this.hub.subscribe(this, "Upload/file", Uju.bind(this, this._squeezeAndQueue));
      this.hub.subscribe(this, "FileWatcher/fileChanged", Uju.bind(this, this._handleModifiedFile));
      this.hub.subscribe(this, "ConnectivityWatcher/connection/recovered", Uju.bind(this, this._onConnectionRecovered));
      this.hub.subscribe(this, "ConnectivityWatcher/connection/lost", Uju.bind(this, this._onConnectionLost));
    },
    start: function() {
      this._resumeFSQueue();
    },
    stop: function() {},

    /*
      * Return the current entries to upload
     */
    getQueueForView: function() {
      var failures, queue, task;
      queue = (function() {
        var _i, _len, _ref2, _results;
        _ref2 = this._queue.items;
        _results = [];
        for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
          task = _ref2[_i];
          _results.push(task.file);
        }
        return _results;
      }).call(this);
      failures = (function() {
        var _i, _len, _ref2, _results;
        _ref2 = this._queue.failures;
        _results = [];
        for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
          task = _ref2[_i];
          _results.push(task.file);
        }
        return _results;
      }).call(this);
      return queue.reverse().concat(failures);
    },

    /*
      * Return the currently uploading entry
     */
    getUploadingForView: function() {
      var _ref2;
      return ((_ref2 = this.uploading) != null ? _ref2.file : void 0) || null;
    },

    /*
      * Set progress callback
      * @param <function> callback or null/undefined
     */
    setProgressTracker: function(callback) {
      if (Uju.isFunction(callback) || (callback == null)) {
        return this.progressTracker = callback;
      } else {
        throw "Upload :: setProgressTracker :: Param is not a function";
      }
    },

    /*
      * On network access recovering
     */
    _onConnectionRecovered: function() {
      var _results;
      if (this.uploading != null) {
        this._checkConflict(this.uploading);
        this._notifyHub(this.getQueueForView(), this.uploading);
      } else {
        this._onUploadDone();
      }
      _results = [];
      while (this._queue.failures.length > 0) {
        _results.push(this._addToQueue(this._queue.failures.splice(0, 1)[0]));
      }
      return _results;
    },
    _onConnectionLost: function() {
      var queue;
      queue = this.getQueueForView();
      if (this.uploading != null) {
        queue.splice(0, 0, this.uploading.file);
      }
      return this._notifyHub(queue, null);
    },

    /*
      * Upload a file modified by a third-party application
      * @param <json> inEvent: event containing the file-to-upload's full path
     */
    _handleModifiedFile: function(inEvent) {
      var idCloudFolder, path, pathArray;
      pathArray = inEvent.file_path.split("/");
      pathArray.splice(0, pathArray.indexOf("NumSync") + 2);
      idCloudFolder = parseInt(pathArray[0]);
      pathArray.splice(0, 1);
      path = pathArray.join("/");
      if (Uju.isNumber(idCloudFolder) && path) {
        return this._squeezeAndQueue({
          filePath: inEvent.file_path,
          file: {
            idCloudFolder: idCloudFolder,
            path: path,
            lastModified: inEvent.modifiedOn
          }
        });
      } else {
        return console.error("Upload :: _handleModifiedFile :: Couldn't upload modified file '" + inEvent.file_path);
      }
    },

    /*
      * Format an event object received from the app to upload a file
      * @param inEvent
     */
    _squeezeAndQueue: function(inEvent) {
      delete inEvent.source;
      delete inEvent.topic;
      inEvent.file.type = NumSync.models.Entry.types.FILE;
      inEvent.file = new NumSync.models.File(inEvent.file);
      return this._addToQueue(inEvent);
    },

    /*
      * Add a file to the uploading queue
      * @param <json> inEvent: See @_uploadFile
     */
    _addToQueue: function(inEvent) {
      var file, inPipe, newUploadingEntry, oldStatus, _i, _len;
      if (inEvent == null) {
        return;
      }
      inPipe = this._queue.items.concat(this._queue.failures);
      if (this.uploading != null) {
        inPipe.push(this.uploading);
      }
      for (_i = 0, _len = inPipe.length; _i < _len; _i++) {
        file = inPipe[_i];
        if (file.idCloudFolder === inEvent.file.idCloudFolder && file.path === inEvent.file.path) {
          console.warn("Upload :: _addToQueue :: File 'file.path' already in queue");
          return;
        }
      }
      if (inEvent.attempts == null) {
        inEvent.attempts = 0;
      }
      oldStatus = this._queue.working;
      this._queue.working = true;
      if (oldStatus === true) {
        this._queue.items.splice(0, 0, inEvent);
      } else {
        newUploadingEntry = inEvent.file;
        this.uploading = inEvent;
        this._checkConflict(inEvent);
      }
      this._notifyHub(this.getQueueForView(), newUploadingEntry);
      return this._persistQueue();
    },

    /*
      * Check remote file version before uploading it
      * @param <json> toUpload: see @_uploadFile
     */
    _checkConflict: function(toUpload) {
      var local;
      local = toUpload.file;
      return this.hFM.getFileInfo(local.idCloudFolder, local.path, {
        onSuccess: Uju.bind(this, function(remote) {
          var fileName;
          if ((local.lastModified == null) || local.lastModified > remote.lastModified) {
            return this._uploadFile(toUpload, true);
          } else {
            fileName = local.path.split("/").pop();
            return navigator.notification.confirm(this.i18n.getTranslation("conflict_msg", [fileName]), Uju.bind(this, function(buttonIndex) {
              if (buttonIndex === 1) {
                return this._uploadFile(toUpload, true);
              } else {
                return this._onUploadDone();
              }
            }), this.i18n.getTranslation("conflict_title"), [this.i18n.getTranslation("yes"), this.i18n.getTranslation("no")]);
          }
        }),
        onError: Uju.bind(this, function(error, status) {
          if (status === 404) {
            return this._uploadFile(toUpload, false);
          } else {
            alert("Error " + status + " on HEAD request to check file conflict '" + (local.path.split("/").pop()) + "'");
            return this._onUploadDone();
          }
        })
      });
    },

    /*
      * Upload a file
      * @param <json> toUpload: {
      *   filePath <FileEntry>
      *   file <File>
      * }
      * @param <boolean> update: set the httpMethod. Default "POST", true for "PUT"
     */
    _uploadFile: function(toUpload, update) {
      var httpMethod, result;
      httpMethod = update === true ? "PUT" : "POST";
      return result = this.hFM.uploadFile(toUpload.filePath, toUpload.file.getPayload(), {
        onSuccess: Uju.bind(this, this._onUploadSuccess, toUpload),
        onError: Uju.bind(this, this._onUploadFail, toUpload),
        onProgress: Uju.bind(this, this._onUploadProgress),
        httpMethod: httpMethod
      });
    },

    /*
      * On upload progress
      * @param <event> progressEvent: See cordova FileTransfer documentation
      * @param <function> abort: Abort the upload
     */
    _onUploadProgress: function(progressEvent, abort) {
      return typeof this.progressTracker === "function" ? this.progressTracker(progressEvent, abort) : void 0;
    },

    /*
      * On upload success/fail
      * Make the queue continue and update widget
      * @param <boolean> success: true if upload was a success
     */
    _onUploadDone: function(success) {
      var newUploadingEntry, next;
      next = this._queue.items.pop();
      if (next != null) {
        this.uploading = next;
        newUploadingEntry = next.file;
        this._checkConflict(next);
      } else {
        this._queue.working = false;
        this.uploading = null;
        newUploadingEntry = null;
      }
      this._notifyHub(this.getQueueForView(), newUploadingEntry);
      return this._persistQueue();
    },

    /*
      * Called when a file is successfully uploaded
      * @param <json> fileWrapper: info on file that has been uploaded
      * @param <json> response: response
     */
    _onUploadSuccess: function(fileWrapper, response) {
      this._onUploadDone();
      return enyo.Signals.send("refreshFilesTree", {});
    },

    /*
      * Called when a file couldn't be uploaded
      * @param <json> fileWrapper: info on file that couldn't be uploaded
      * @param <json> error: error object
     */
    _onUploadFail: function(fileWrapper, error) {
      var message, retrying;
      if (error.code === FileTransferError.FILE_NOT_FOUND_ERR) {
        retrying = false;
      } else {
        if (!error.http_status || error.http_status === 500) {
          fileWrapper.attempts++;
          retrying = true;
        } else {
          retrying = false;
        }
      }
      if (retrying) {
        message = this.i18n.getTranslation("upload_fail", [fileWrapper.file.path]) + this.i18n.getTranslation("upload_retrying", [fileWrapper.attempts]);
      } else {
        message = this.i18n.getTranslation("upload_canceled", [fileWrapper.file.path]);
      }
      enyo.Signals.send("showModal", {
        clean: false,
        modal: {
          kind: "NumSync.Notification",
          type: NumSync.Notification.types.ERROR,
          message: message,
          timeout: 3000
        }
      });
      if (retrying) {
        this._queue.failures.push(fileWrapper);
        setTimeout(Uju.bind(this, function() {
          var index;
          if (this.hCW.hasConnection()) {
            index = this._queue.failures.indexOf(fileWrapper);
            if (index !== -1) {
              return this._addToQueue(this._queue.failures.splice(index, 1)[0]);
            } else {
              return console.warn("Upload :: retrying :: FileWrapper not found in failures");
            }
          }
        }), 15000);
      }
      return this._onUploadDone();
    },

    /*
      * Publish event on the hub
      * @param <array> [optional] queue: the entries awaiting upload. If undefined or null, don't update
      * @param <Entry/null> [optional] uploading: the uploading entry. If null means nothing is uploading, If undefined means don't update
     */
    _notifyHub: function(queue, uploading) {
      return this.hub.publish(this, "Upload/updateView", {
        uploading: uploading,
        queue: queue
      });
    },

    /*
      * Save the queue in FileSystem
     */
    _persistQueue: function() {
      return this.hFM.saveUploadQueue(this.uploading, this._queue.items, this._queue.failures);
    },

    /*
      * Resume saved queue
     */
    _resumeFSQueue: function() {
      return this.hFM.getUploadQueue({
        onSuccess: Uju.bind(this, function(uploading, queue, failures) {
          var task, _i, _j, _len, _len1;
          if (uploading != null) {
            this._squeezeAndQueue(uploading);
          }
          if (queue != null) {
            for (_i = 0, _len = queue.length; _i < _len; _i++) {
              task = queue[_i];
              this._squeezeAndQueue(task);
            }
          }
          if (failures != null) {
            for (_j = 0, _len1 = failures.length; _j < _len1; _j++) {
              task = failures[_j];
              this._squeezeAndQueue(task);
            }
          }
          if ((uploading != null) || ((queue != null) && queue.length) || ((failures != null) && failures.length)) {
            return enyo.Signals.send("showModal", {
              modal: {
                kind: "Ubidreams.widgets.Modals",
                userClosable: false,
                classes: "modals-notifications",
                components: [
                  {
                    tag: "i",
                    classes: "fa fa-info-circle"
                  }, {
                    tag: "span",
                    content: this.i18n.getTranslation("resuming_uploads")
                  }
                ],
                scrim: false,
                handlers: {
                  onOpen: "_onOpen"
                },
                _onOpen: function() {
                  return setTimeout(Uju.bind(this, function() {
                    return this.close(true);
                  }), 3000);
                }
              }
            });
          }
        }),
        onError: function(error) {
          console.error("Upload :: _resumeFSQueue :: Couldn't resume queue: ");
          return console.error(error);
        }
      });
    }
  };

}).call(this);
