
/*
  # Copyright ® 2008-2013, Ubiquitous Dreams SARL - SIREN 508 175 437 RCS La Rochelle.
  # All rights reserved.
  # 1 rue Fleming, 17000 - La Rochelle, France.
*/


(function() {
  var scope, _ref, _ref1, _ref2;

  scope = typeof window !== "undefined" && window !== null ? window : this;

  scope.Ubidreams = (_ref = scope.Ubidreams) != null ? _ref : {};

  scope.Ubidreams.widgets = (_ref1 = scope.Ubidreams.widgets) != null ? _ref1 : {};

  scope.Ubidreams.widgets.Abstract = (_ref2 = scope.Ubidreams.widgets.Abstract) != null ? _ref2 : {};

  scope.Ubidreams.widgets.Abstract.contract = {
    drawWidget: function(container) {},
    addSubWidget: function(subWidget) {},
    removeSubWidget: function(subWidget) {},
    getName: function() {},
    getFieldName: function() {}
  };

  scope.Ubidreams.widgets.Abstract.impl = (function() {

    impl.prototype.CONTAINER_DOM = 0;

    impl.prototype.CONTAINER_ENYO = 1;

    function impl() {
      this.hub = void 0;
      this.widget = void 0;
      this.ui = void 0;
      this.noInit = false;
      this.container = void 0;
      this.containerType = void 0;
      this.emptyContainer = true;
      this.widgetConf = {};
      this.field = null;
      this.subWidgets = [];
      this.registeredSubWidgets = {};
      this.scrollTo = void 0;
    }

    /*  H-UBU COMPONENT IMPLEMENTATION
    */


    impl.prototype.getComponentName = function() {
      return this.name;
    };

    impl.prototype.getName = function() {
      return this.getComponentName();
    };

    impl.prototype.configure = function(hub, configuration) {
      var key, val, _ref3;
      this.hub = hub;
      try {
        if ((configuration == null) || ((configuration != null) && !((configuration.field != null) || (configuration.container != null) || configuration.noInit === true || this.noInit === true))) {
          throw "You must at least specify a 'container' or a 'field' in the configuration object, or set 'noInit' to true";
        }
        if (configuration.name == null) {
          this.name = Uju.createUUID();
        } else {
          this.name = configuration.name;
        }
        if (!(configuration.container == null)) {
          this._setContainer(configuration.container);
        }
        if (!(configuration.emptyContainer == null)) {
          this.emptyContainer = configuration.emptyContainer;
        }
        if (!(configuration.field == null)) {
          this.field = configuration.field;
        }
        if (!(configuration.widgetConf == null)) {
          this.widgetConf = configuration.widgetConf;
        }
        if (!(configuration.noInit == null)) {
          this.noInit = configuration.noInit;
        }
        if (!(configuration.scrollTo == null)) {
          this.scrollTo = configuration.scrollTo;
        }
        if (!this.contract) {
          this.contract = scope.Ubidreams.widgets.Abstract.contract;
        } else {
          _ref3 = scope.Ubidreams.widgets.Abstract.contract;
          for (key in _ref3) {
            val = _ref3[key];
            this.contract[key] = val;
          }
        }
        this.hub.provideService({
          component: this,
          contract: this.contract
        });
      } catch (e) {
        console.error("AbstractWidget :: configure :: Error: " + e);
      }
    };

    impl.prototype.start = function() {
      this._setUpWidgetController();
      this.widget = eval("new " + this.ui + "(this.widgetConf)");
      if (!this.noInit) {
        this.drawWidget();
      }
      this._registerSubWidgets();
    };

    impl.prototype.stop = function() {
      var _ref3;
      this._unregisterSubWidgets();
      if (((_ref3 = this.widget) != null ? _ref3.destroy : void 0) != null) {
        this.widget.destroy();
        this.widget = null;
      }
      if (scope.debugNavigation === true) {
        console.debug("Component with ui " + this.ui + " sucessfully unregistered");
      }
    };

    /*
     # Renders the enyo widget into the DOM element with id equals to @container
     #
     # @param container: (Optional) <String> A DOM element id. If specified, it will override @container
    */


    impl.prototype.drawWidget = function(container) {
      var tmpDiv;
      if (container != null) {
        this._setContainer(container);
      }
      if (this.container == null) {
        throw "AbstractWidget :: drawWidget :: No container to draw the widget in.";
      }
      switch (this.containerType) {
        case this.CONTAINER_DOM:
          if (this.emptyContainer === true) {
            this.widget.renderInto(this.container);
          } else {
            tmpDiv = document.createElement("div");
            tmpDiv.innerHTML = this.widget.generateHtml();
            this.container.appendChild(tmpDiv.firstChild);
            this.widget.render();
          }
          break;
        case this.CONTAINER_ENYO:
          if (this.emptyContainer === true) {
            this.container.destroyClientControls();
          }
          this.widget.setContainer(this.container);
          this.widget.setOwner(this.container);
          this.widget.render();
      }
      if (this.scrollTo != null) {
        $('html, body').animate({
          scrollTop: $("#" + this.scrollTo).offset().top
        }, 'slow');
      } else {
        $('html, body').animate({
          scrollTop: 0
        }, 'fast');
      }
    };

    /*
     # Add a subwidget to this widget
     #
     # @param subWidget a json config object of the subwidget to add
    */


    impl.prototype.addSubWidget = function(subWidget) {
      var comp, field, props, proxy;
      if (subWidget.config == null) {
        subWidget.config = {};
      }
      if (subWidget.config.noInit == null) {
        subWidget.config.noInit = true;
      }
      if (subWidget.config.aggregate == null) {
        subWidget.config.aggregate = false;
      }
      comp = eval("new " + subWidget.className + ".impl()");
      this.hub.registerComponent(comp, subWidget.config);
      if (subWidget.config.field != null) {
        field = subWidget.config.field;
        proxy = {};
        proxy.__proxy__ = comp;
        for (props in comp.contract) {
          if (Uju.isFunction(comp.contract[props])) {
            proxy[props] = Uju.bind(comp, comp[props]);
          } else {
            proxy[props] = comp[props];
          }
        }
        if (!subWidget.config.aggregate) {
          return this.registeredSubWidgets[field] = proxy;
        } else {
          if (this.registeredSubWidgets[field] == null) {
            this.registeredSubWidgets[field] = [];
          }
          return this.registeredSubWidgets[field].push(proxy);
        }
      }
    };

    /*
    # Remove a subwidget from this widget
    #
    # @param subWidget String the field where the to-be-removed widget is stored in @registeredSubWidgets
    */


    impl.prototype.removeSubWidget = function(subWidget) {
      var comp, component, _i, _len;
      if (!Uju.isString(subWidget)) {
        console.error("AbstractWidget :: removeSubWidget :: Error: Param 'subWidget' must be a string");
        return;
      }
      component = this.registeredSubWidgets[subWidget];
      if (component != null) {
        if (this._isSubWidget(component)) {
          if (Uju.isArray(component)) {
            for (_i = 0, _len = component.length; _i < _len; _i++) {
              comp = component[_i];
              this.hub.unregisterComponent(comp.getName());
            }
          } else {
            this.hub.unregisterComponent(component.getName());
          }
          delete this.registeredSubWidgets[subWidget];
          if (this.registeredSubWidgets[subWidget] != null) {
            this.registeredSubWidgets[subWidget] = null;
          }
          return;
        }
      }
      console.warn("AbstractWidget :: removeSubWidget :: Couldn't find subwidget ");
      console.warn(subWidget);
    };

    /*
     # Return the field name where the subwidget is stored in, or null
    */


    impl.prototype.getFieldName = function() {
      return this.field;
    };

    /*
      * Set the container for pages
      * @param container: <string|DOM element|enyo instance>
    */


    impl.prototype._setContainer = function(container) {
      var element, type;
      element = null;
      if (Uju.isString(container)) {
        element = container !== "body" ? document.getElementById(container) : document.body;
        type = this.CONTAINER_DOM;
      } else if (this._isElement(container)) {
        element = container;
        type = this.CONTAINER_DOM;
      } else if (this._isEnyoObject(container)) {
        element = container;
        type = this.CONTAINER_ENYO;
      }
      if (element == null) {
        throw "Navigation :: setContainer :: Container is not a valid DOM element (or id) or a valid enyo object.";
      }
      this.container = element;
      return this.containerType = type;
    };

    /*
     # Return true if the param comp is a subwidget of the current instance
     #
     # @param comp The component to check
    */


    impl.prototype._isSubWidget = function(comp) {
      var key;
      for (key in this.registeredSubWidgets) {
        if (comp === this.registeredSubWidgets[key]) {
          return true;
        }
      }
      return false;
    };

    /*
    # Register sub widgets on the hub
    */


    impl.prototype._registerSubWidgets = function() {
      var subWidget, _i, _len, _ref3, _results;
      _ref3 = this.subWidgets;
      _results = [];
      for (_i = 0, _len = _ref3.length; _i < _len; _i++) {
        subWidget = _ref3[_i];
        _results.push(this.addSubWidget(subWidget));
      }
      return _results;
    };

    /*
    # Unregister sub widgets from the hub
    */


    impl.prototype._unregisterSubWidgets = function() {
      var key;
      for (key in this.registeredSubWidgets) {
        this.removeSubWidget(key);
      }
      return this.registeredSubWidgets = {};
    };

    /*
    # Add some properties to the widget conf so that the view (widget) has access to the controller's methods
    */


    impl.prototype._setUpWidgetController = function() {
      this.widgetConf.__controller__ = this;
      return this.widgetConf._call = Uju.bind(this, function(functionName) {
        var args, i, _i, _ref3;
        args = [];
        for (i = _i = 1, _ref3 = arguments.length; 1 <= _ref3 ? _i < _ref3 : _i > _ref3; i = 1 <= _ref3 ? ++_i : --_i) {
          args.push(arguments[i]);
        }
        return this[functionName].apply(this, args);
      });
    };

    impl.prototype._isElement = function(o) {
      if (typeof HTMLElement === "object") {
        return o instanceof HTMLElement;
      }
      return (o != null) && Uju.isObject(o) && o.nodeType === 1 && Uju.isString(o.nodeName);
    };

    impl.prototype._isEnyoObject = function(o) {
      return o.hasNode != null;
    };

    return impl;

  })();

}).call(this);
;

/*
  # Copyright ® 2008-2013, Ubiquitous Dreams SARL - SIREN 508 175 437 RCS La Rochelle.
  # All rights reserved.
  # 1 rue Fleming, 17000 - La Rochelle, France.
*/


(function() {
  var scope, _ref, _ref1;

  scope = typeof window !== "undefined" && window !== null ? window : this;

  scope.Ubidreams = (_ref = scope.Ubidreams) != null ? _ref : {};

  scope.Ubidreams.Navigation = (_ref1 = scope.Ubidreams.Navigation) != null ? _ref1 : {};

  scope.Ubidreams.Navigation.contract = {
    loadPage: function(page) {},
    setUpNavigation: function() {},
    getCurrentPage: function() {},
    setContainer: function(container) {},
    init: function() {}
  };

  scope.Ubidreams.Navigation.impl = {
    /* LOCAL ATTRIBUTES
    */

    hub: void 0,
    namespaceStr: void 0,
    namespace: void 0,
    container: void 0,
    homePage: "home",
    html5Nav: false,
    currentPage: {},
    onInternalLinkClick: void 0,
    autoInit: true,
    /*  H-UBU COMPONENT IMPLEMENTATION
    */

    getComponentName: function() {
      return "Navigation";
    },
    configure: function(hub, configuration) {
      var namespace, subObject, tree, _i, _len, _ref2;
      this.hub = hub;
      try {
        if ((configuration != null ? configuration.namespace : void 0) == null) {
          throw "You must specify a namespace where to get page components";
        }
        this.container = configuration.container;
        if (!(configuration.init == null)) {
          this.autoInit = configuration.init;
        }
        this.namespaceStr = configuration.namespace;
        if (!(configuration.homePage == null)) {
          this.homePage = configuration.homePage;
        }
        if (!(configuration.html5Nav == null)) {
          this.html5Nav = configuration.html5Nav;
        }
        scope.debugNavigation = (_ref2 = configuration.debug) != null ? _ref2 : false;
        tree = this.namespaceStr.split(".");
        namespace = scope;
        for (_i = 0, _len = tree.length; _i < _len; _i++) {
          subObject = tree[_i];
          namespace = namespace[subObject];
          if (namespace == null) {
            throw "Navigation :: configure :: The given namespace (" + this.namespaceStr + ") is invalid. Error happened at the '" + subObject + "' part of it.";
          }
        }
        this.namespace = namespace;
        this.hub.provideService({
          component: this,
          contract: scope.Ubidreams.Navigation.contract
        });
        this.hub.subscribe(this, "Navigation/loadPage", this.loadPage);
        this.hub.subscribe(this, "Navigation/setUpNavigation", this.setUpNavigation);
        return this.hub.subscribe(this, "Navigation/getCurrentPage", Uju.bind(this, function(inEvent) {
          return inEvent.callback(this.getCurrentPage());
        }));
      } catch (e) {
        return console.error("Navigation :: configure :: Error: " + e);
      }
    },
    start: function() {
      if (this.autoInit) {
        this.init();
      }
    },
    stop: function() {},
    /*
      * Set the container for pages
      * @param container: <DOM element id|DOM element|enyo instance>
    */

    setContainer: function(container) {
      return this.container = container;
    },
    /*
      * Init the navigation
    */

    init: function() {
      var handler, page;
      if (this.container == null) {
        throw "Navigation :: init :: No container is set.";
      }
      if (this.html5Nav === true) {
        handler = Uju.bind(this, function(e) {
          var _ref2;
          return this.loadPage({
            page: (_ref2 = this._retrievePageFromUrl()) != null ? _ref2 : this.homePage,
            preventReload: true
          });
        });
        if ((window.addEventListener != null)) {
          window.addEventListener('popstate', handler);
        } else if ((window.attachEvent != null)) {
          window.attachEvent('onpopstate', handler);
        } else {
          window['onpopstate'] = handler;
        }
        page = this._retrievePageFromUrl();
      }
      return this.loadPage({
        page: page != null ? page : this.homePage
      });
    },
    /*
     # Load a page component
     #
     # @param inEvent: {
     #   page: Name of the page to load. A page has a name and a component. For example page named "test" will have a component TestPage
     #   data: An object to pass to the page component on its registration on the hub
     #   scrollTo: A DOM id to scroll to once the page is loaded
     #   preventReload: Boolean, if true, the page won't load if it is the current page
     #   callback: a callback to call when page is loaded (with a boolean param, true if the page as loaded, false otherwise)
     # }
    */

    loadPage: function(inEvent) {
      var cmp, compName, data, page, scrollTo, _ref2, _ref3;
      if (scope.debugNavigation === true) {
        console.debug("Navigation component :: loadPage :: loading page " + inEvent.page);
      }
      page = inEvent.page;
      data = inEvent.data;
      scrollTo = inEvent.scrollTo;
      if (inEvent.preventReload === true && page === this.currentPage.pageName) {
        if (typeof inEvent.callback === "function") {
          inEvent.callback(false);
        }
        if (scope.debugNavigation === true) {
          console.debug("Navigation component :: loadPage :: The page reloading has been prevented");
        }
        return;
      }
      compName = page.charAt(0).toUpperCase() + page.slice(1) + "Page";
      this.hub.publish(this, "Navigation/pageChanging", {
        page: page
      });
      if (((_ref2 = this.namespace[compName]) != null ? _ref2.impl : void 0) != null) {
        if (this.currentPage.compName != null) {
          this.hub.unregisterComponent(this.currentPage.compName);
        }
        if (this.html5Nav === true && (((_ref3 = window.history) != null ? _ref3.pushState : void 0) != null) && Uju.isFunction(window.history.pushState)) {
          window.history.pushState(null, null, "/#!page=" + page + (data != null ? "&data=" + data : ""));
        }
        cmp = Uju.isFunction(this.namespace[compName].impl) ? new this.namespace[compName].impl() : cmp = this.namespace[compName].impl;
        this.hub.registerComponent(cmp, {
          container: this.container,
          data: data,
          scrollTo: scrollTo
        });
        this.currentPage.compName = cmp.getComponentName();
        this.currentPage.pageName = page;
        this.setUpNavigation();
        this.hub.publish(this, "Navigation/pageChanged", {
          page: page
        });
        return typeof inEvent.callback === "function" ? inEvent.callback(true) : void 0;
      } else {
        if (typeof inEvent.callback === "function") {
          inEvent.callback(false);
        }
        if (page === "home") {
          document.body.innerHTML = "<br/><center><h1>An error has occured, please try again later.</h1></center>";
          console.error("Navigation :: loadPage :: Can't load Home page: The page or its 'impl' property does not exist");
        } else {
          console.warn("Navigation :: Page component 'scope." + this.namespaceStr + "." + compName + ".impl' does not exist. Loading home page.");
          this.loadPage({
            page: "home",
            callback: inEvent.callback
          });
        }
      }
    },
    /*
     # Set up intern navigation links. This function will scan every DOM element on the browser page
     # To create a link just set the 'data-page' attributes to the page name to be loaded on click.
     # This function is called each time a page is loaded (loadPage). So you must call it if you add a navigation link dynamically without loading a page
    */

    setUpNavigation: function() {
      var component, internalLinks;
      internalLinks = $('*[data-page]');
      internalLinks.each(function(index) {
        if ($(this)[0].tagName === "A") {
          return $(this).attr("href", "#!page=" + $(this).attr("data-page"));
        }
      });
      component = this;
      if (this.onInternalLinkClick == null) {
        this.onInternalLinkClick = function(e) {
          e.preventDefault();
          return component.loadPage({
            page: $(this).attr("data-page"),
            scrollTo: $(this).attr("data-page-scrollto")
          });
        };
      }
      internalLinks.off("click", this.onInternalLinkClick);
      return internalLinks.on("click", this.onInternalLinkClick);
    },
    /*
     # Return the name of the current page or undefined
    */

    getCurrentPage: function() {
      return this.currentPage.pageName;
    },
    /*
     # Return the page parameter in the URL, or null
    */

    _retrievePageFromUrl: function() {
      var hash, param, params, _i, _len, _ref2;
      hash = (_ref2 = location.href.split("#!")[1]) != null ? _ref2 : "";
      params = hash.split("&");
      for (_i = 0, _len = params.length; _i < _len; _i++) {
        param = params[_i];
        param = param.split("=");
        if (param[0] === "page") {
          return param[1];
        }
      }
      return null;
    },
    _isElement: function(o) {
      if (typeof HTMLElement === "object") {
        return o instanceof HTMLElement;
      }
      return (o != null) && Uju.isObject(o) && o.nodeType === 1 && Uju.isString(o.nodeName);
    },
    _isEnyoObject: function(o) {
      return Uju.isObject(o) && (o.kind != null) && (o.render != null);
    }
  };

}).call(this);
;
