(function() {
  var scope, _ref;

  scope = typeof window !== "undefined" && window !== null ? window : this;

  /**
    @namespace AjaxService
  */


  scope.AjaxService = (_ref = scope.AjaxService) != null ? _ref : {};

  /**
   HAjaxService contract
  
   Base inParams structure
   {
      url: ""                //(REQUIRED) URL where to send the request (ex: http://twitter.com)
      params: {...}          //Additional parameters to send in the query. For instance twitter need a query param: {q: "Text to search"}
      onSuccess: func        //callback called upon success of this request
      onError: func          //callback called upon failure of the request
      additionalconf: {...}  //Other parameters depends on implementation //TODO Normalize optional additionalconf through implems
   }
  
   Arguments order in callbacks :
     onSuccess : data, textStatus, jqXHR
     onError : jqXHR, textStatus, errorThrown
   @class AjaxService.contract
  */


  scope.AjaxService.contract = {
    del: function() {},
    get: function() {},
    post: function() {},
    postJSON: function() {},
    put: function() {},
    putJSON: function() {},
    getJSONP: function() {},
    ajax: function() {}
  };

}).call(this);
;
;
(function() {
  var scope, _ref, _ref1;

  scope = typeof window !== "undefined" && window !== null ? window : this;

  /**
    @namespace AjaxService
  */


  scope.AjaxService = (_ref = scope.AjaxService) != null ? _ref : {};

  /**
    @namespace AjaxService.impls
  */


  scope.AjaxService.impls = (_ref1 = scope.AjaxService.impls) != null ? _ref1 : {};

  /**
    @class AjaxService.impls.abstract
  */


  scope.AjaxService.impls.abst = (function() {

    abst.prototype._hub = null;

    abst.prototype._name = null;

    abst.prototype._logger = null;

    function abst(name) {
      this._isJsonpSpecial = false;
      if ((name == null) || name === "") {
        name = "AjaxService";
      }
      this._name = name;
      return;
    }

    abst.prototype.getComponentName = function() {
      return this._name;
    };

    abst.prototype.configure = function(inHub, inConfig) {
      this._hub = inHub;
      this._logger = new Logger(this.getComponentName());
      if (this._checkRequirements()) {
        this._hub.provideService({
          component: this,
          contract: scope.AjaxService.contract
        });
      } else {
        this._logger.error("" + (this.getComponentName()) + " Unable to provide AjaxService.contract, requirements not met");
      }
    };

    abst.prototype.start = function() {};

    abst.prototype.stop = function() {};

    abst.prototype.get = function(inParams) {
      inParams.method = "GET";
      this.ajax(inParams);
    };

    abst.prototype.post = function(inParams) {
      inParams.method = "POST";
      this.ajax(inParams);
    };

    abst.prototype.put = function(inParams) {
      inParams.method = "PUT";
      this.ajax(inParams);
    };

    abst.prototype.del = function(inParams) {
      inParams.method = "DELETE";
      this.ajax(inParams);
    };

    abst.prototype.postJSON = function(inParams) {
      if ((inParams != null ? inParams.payload : void 0) != null) {
        inParams.payload = JSON.stringify(inParams.payload);
      }
      inParams.contentType = "application/json";
      return this.post(inParams);
    };

    abst.prototype.putJSON = function(inParams) {
      if ((inParams != null ? inParams.payload : void 0) != null) {
        inParams.payload = JSON.stringify(inParams.payload);
      }
      inParams.contentType = "application/json";
      return this.put(inParams);
    };

    abst.prototype.ajax = function(inParams) {
      var request;
      if (!this._areParamsValid(inParams)) {
        return;
      }
      request = this._paramsToRequest(inParams);
      return this._sendRequest(request);
    };

    abst.prototype._areParamsValid = function(inParams) {
      if ((inParams == null) || (inParams.url == null) || (inParams.method == null)) {
        this._logger.error("" + (this.getComponentName()) + " invalid parameters : " + inParams);
        return false;
      } else {
        if (inParams.onSuccess == null) {
          this._logger.warn("" + (this.getComponentName()) + " no success callback set");
        }
        if (inParams.onError == null) {
          this._logger.warn("" + (this.getComponentName()) + " no error callback set");
        }
        return true;
      }
    };

    return abst;

  })();

}).call(this);
;
;
(function() {
  var scope, _ref, _ref1,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  scope = typeof window !== "undefined" && window !== null ? window : this;

  /**
    @namespace AjaxService
  */


  scope.AjaxService = (_ref = scope.AjaxService) != null ? _ref : {};

  /**
    @namespace AjaxService.impls
  */


  scope.AjaxService.impls = (_ref1 = scope.AjaxService.impls) != null ? _ref1 : {};

  /**
    @class AjaxService.impls.jquery
  */


  scope.AjaxService.impls.jquery = (function(_super) {

    __extends(jquery, _super);

    function jquery(name) {
      $.support.cors = true;
      if ((name == null) || name === "") {
        name = "AjaxService-jquery";
      }
      jquery.__super__.constructor.call(this, name);
    }

    jquery.prototype.getJSONP = function(inParams) {
      var foo,
        _this = this;
      inParams.responseType = "jsonp";
      if (inParams.onSuccess != null) {
        foo = inParams.onSuccess;
        inParams.onSuccess = function(data, error, jqXHR) {
          if (Uju.isString(data)) {
            data = JSON.parse(data);
          }
          return foo(data, error, jqXHR);
        };
      }
      return this.get(inParams);
    };

    jquery.prototype._checkRequirements = function() {
      return typeof jQuery !== "undefined" && jQuery !== null;
    };

    jquery.prototype._paramsToRequest = function(inParams) {
      var request, _ref2;
      request = (_ref2 = inParams.more) != null ? _ref2 : {};
      request.url = inParams.url;
      if (inParams.params != null) {
        request.url = Uju.buildUrl(request.url, inParams.params);
      }
      request.type = inParams.method;
      if (inParams.onSuccess != null) {
        request.success = inParams.onSuccess;
      }
      if (inParams.onError != null) {
        request.error = inParams.onError;
      }
      if (inParams.onComplete != null) {
        request.complete = inParams.onComplete;
      }
      if (inParams.contentType != null) {
        request.contentType = inParams.contentType;
      }
      if (inParams.payload != null) {
        request.data = inParams.payload;
      }
      if (inParams.responseType != null) {
        request.dataType = inParams.dataType;
      }
      return request;
    };

    jquery.prototype._sendRequest = function(request) {
      return $.ajax(request);
    };

    return jquery;

  })(scope.AjaxService.impls.abst);

}).call(this);
;
