(function() {
  var scope, _ref, _ref1;

  scope = typeof window !== "undefined" && window !== null ? window : this;

  scope.Ubidreams = (_ref = scope.Ubidreams) != null ? _ref : {};

  scope.Ubidreams.i18n = (_ref1 = scope.Ubidreams.i18n) != null ? _ref1 : {};

  scope.Ubidreams.i18n.contract = {
    getTranslation: function(key) {},
    getLanguage: function() {},
    setLanguage: function(_language) {},
    addI18nData: function(_data) {},
    i18n: function(_i18nKeys, _onLanguageChange) {},
    i18nForEnyo: function(_enyoObject) {},
    i18nForDOM: function(_DOMnode) {},
    clearTriggers: function() {}
  };

  scope.Ubidreams.i18n.impl = {
    hub: null,
    i18n_data: {},
    language: null,
    i18n_triggers: {},
    getComponentName: function() {
      return 'i18n';
    },
    /*
      * Configuration
      *
      * configuration = {
      *   language:String
      * }
    */

    configure: function(hub, configuration) {
      this.hub = hub;
      this.i18n_data = {};
      this.i18n_triggers = {};
      if ((configuration != null ? configuration.language : void 0) != null) {
        this.language = configuration.language;
      } else {
        this._getCurrentLanguage();
      }
      return this.hub.provideService({
        component: this,
        contract: scope.Ubidreams.i18n.contract
      });
    },
    start: function() {
      if (this.language == null) {
        return this._getCurrentLanguage();
      } else {
        return this._executeI18nTriggers();
      }
    },
    stop: function() {},
    /*
      * Get Language
    */

    getLanguage: function() {
      return this.language;
    },
    /*
      * Set language
      * @_language : String
    */

    setLanguage: function(_language) {
      this.language = _language;
      this._executeI18nTriggers();
      return this.hub.publish(this, "Language/onChange", {});
    },
    /*
      * Add i18n data (merge with older data)
      * @_data : Array of JSON or JSON item
    */

    addI18nData: function(_data) {
      var key, lang, translations, value, _results;
      _results = [];
      for (key in _data) {
        translations = _data[key];
        _results.push((function() {
          var _results1;
          _results1 = [];
          for (lang in translations) {
            value = translations[lang];
            if (this.i18n_data[lang] == null) {
              this.i18n_data[lang] = {};
            }
            _results1.push(this.i18n_data[lang][key] = value);
          }
          return _results1;
        }).call(this));
      }
      return _results;
    },
    /*
      * Calls i18n and return i18n data for current language
      * @_i18nKeys : Array of String or JSON, i18n keys to retrieve translations for current language
      *              /!\ If it's a JSON object translations keys are retrieved from the value in a pair of key/value
      * @_onLanguageChange(_translations) : function, triggered when language has changed with translations data as parameters for current language
      *                                     (in the same order as the given keys array)
    */

    i18n: function(_i18nKeys, _onLanguageChange) {
      var _trigger;
      _trigger = Uju.bind(this, function(keys_, callback_) {
        var key, translationAsJSON, value, _i, _len, _ref2, _ref3, _results, _translationsAsArray;
        if (Uju.isObject(keys_)) {
          translationAsJSON = {};
          for (key in keys_) {
            value = keys_[key];
            translationAsJSON[key] = (_ref2 = this.i18n_data[this.language]) != null ? _ref2[value] : void 0;
          }
          return callback_(translationAsJSON);
        } else {
          _results = [];
          for (_i = 0, _len = keys_.length; _i < _len; _i++) {
            key = keys_[_i];
            _translationsAsArray = [];
            _translationsAsArray.push((_ref3 = this.i18n_data[this.language]) != null ? _ref3[key] : void 0);
            _results.push(callback_(_translationsAsArray));
          }
          return _results;
        }
      }, _i18nKeys, _onLanguageChange);
      return this._addI18nTrigger(_trigger);
    },
    /*
      * Calls i18n for an enyo object and all of its childs
      * It automatically sets i18n data calling setters methods of enyo object
      * Retrieves the i18n property of an enyo object
      * => i18n:[
      *     {key:"String, i18n key to retrieve translation", setter:"String property to change with enyo setter", params:"Array, parameters to bind with setter"}
      *     ...
      *   ]
      * @_enyoObject : an enyo object
    */

    i18nForEnyo: function(_enyoObject, _recursive) {
      return this._scanEnyoObject(_enyoObject, _recursive);
    },
    /*
      * Calls i18n for a DOM node and all of its childs
      * Retrieves the "data-i18n" attribute as a set of key:value separated with whitespace :
      *   -> key : i18n key to retrieve translation
      *   -> value : an HTML attribute using HTML setAttribute method, if value is "content" translation will be applied as innerHTML
      * ex : data-i18n = "mykey:src mykey:content otherkey:title"
      * @_DOMnode : a DOM node
    */

    i18nForDOM: function(_DOMnode) {
      return this._scanDOMnode(_DOMnode);
    },
    /*
      * Clear memorized triggers
    */

    clearTriggers: function() {
      return this.i18n_triggers = {};
    },
    /*
      * Retrieves current language from browser
      * If current language cannot be retrieved, keeps default language
    */

    _getCurrentLanguage: function() {
      var ln;
      ln = (typeof navigator !== "undefined" && navigator !== null ? navigator.language : void 0) || (typeof navigator !== "undefined" && navigator !== null ? navigator.userLanguage : void 0) || "en";
      if (ln.length > 2) {
        ln = ln.slice(0, 2);
      }
      if (ln !== 'en' && ln !== 'fr') {
        ln = "en";
      }
      return this.setLanguage(ln);
    },
    /*
      * Add an new i18n trigger
      * @_trigger : function, trigger to add
      * @return : String, UUID
    */

    _addI18nTrigger: function(_trigger) {
      var id;
      id = Uju.createUUID();
      this.i18n_triggers[id] = _trigger;
      _trigger();
      return id;
    },
    /*
      * Execute all i18n triggers
    */

    _executeI18nTriggers: function() {
      var i18nTrigger, key, _ref2, _results;
      _ref2 = this.i18n_triggers;
      _results = [];
      for (key in _ref2) {
        i18nTrigger = _ref2[key];
        try {
          _results.push(i18nTrigger());
        } catch (error) {
          _results.push(delete this.i18n_triggers[key]);
        }
      }
      return _results;
    },
    /*
      * Scanning enyo object to get components which has "i18n" property
      * For each, creates a new trigger in global triggers object to apply translation depending on current language
      * @_enyoObject : an enyo object
    */

    _scanEnyoObject: function(_enyoObject, _recursive) {
      var child, _i, _len, _ref2, _results, _trigger;
      if (_recursive == null) {
        _recursive = true;
      }
      if (_enyoObject.i18n != null) {
        _trigger = Uju.bind(this, function(enyoObject_) {
          var i18n, params_, setterStr, setter_, translation_, _i, _len, _ref2, _ref3, _results;
          _ref2 = enyoObject_.i18n;
          _results = [];
          for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
            i18n = _ref2[_i];
            setterStr = (_ref3 = i18n.setter) != null ? _ref3 : "content";
            setter_ = "set" + setterStr.charAt(0).toUpperCase() + setterStr.slice(1);
            translation_ = this.getTranslation(i18n.key, i18n.variables);
            if (translation_ != null) {
              params_ = [];
              if (i18n.params != null) {
                params_ = params_.concat(i18n.params);
              }
              params_.push(translation_);
              _results.push(enyoObject_[setter_].apply(enyoObject_, params_));
            } else {
              _results.push(void 0);
            }
          }
          return _results;
        }, _enyoObject);
        this._addI18nTrigger(_trigger);
      }
      if (_recursive === true) {
        _ref2 = _enyoObject.children;
        _results = [];
        for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
          child = _ref2[_i];
          _results.push(this._scanEnyoObject(child));
        }
        return _results;
      }
    },
    /*
      * Scanning DOM node to get nodes which has "data_i18n" attribute
      * For each, creates a new trigger in global triggers object to apply translation depending on current language
      * @_DOMnode : a DOM node
    */

    _scanDOMnode: function(_DOMnode) {
      var child, i18n, _i, _len, _ref2, _results, _trigger;
      i18n = _DOMnode.getAttribute("data-i18n");
      if (i18n) {
        _trigger = Uju.bind(this, function(DOMnode_, i18n_) {
          var entry, entrySet, i18nSet, key, translation_, value, _i, _len, _ref2, _results;
          i18nSet = i18n_.split(" ");
          _results = [];
          for (_i = 0, _len = i18nSet.length; _i < _len; _i++) {
            entry = i18nSet[_i];
            entrySet = entry.split(":");
            key = entrySet[0];
            value = entrySet[1];
            translation_ = (_ref2 = this.i18n_data[this.language]) != null ? _ref2[key] : void 0;
            if (translation_ != null) {
              if (value === "content") {
                _results.push(DOMnode_.innerHTML = translation_);
              } else {
                _results.push(DOMnode_.setAttribute(value, translation_));
              }
            } else {
              _results.push(void 0);
            }
          }
          return _results;
        }, _DOMnode, i18n);
        this._addI18nTrigger(_trigger);
      }
      _ref2 = _DOMnode.children;
      _results = [];
      for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
        child = _ref2[_i];
        _results.push(this._scanDOMnode(child));
      }
      return _results;
    },
    /*
      * Get a translated key in current language
      * @param key: <string> the key to translate
      * @param variables: <array of string> values for key variables
    */

    getTranslation: function(key, variables) {
      var i, str, var_, _i, _len, _ref2;
      str = (_ref2 = this.i18n_data[this.language]) != null ? _ref2[key] : void 0;
      if ((str != null) && Uju.isArray(variables)) {
        for (i = _i = 0, _len = variables.length; _i < _len; i = ++_i) {
          var_ = variables[i];
          str = str.replace("{{" + i + "}}", var_);
        }
      }
      return str;
    }
  };

  enyo.kind({
    name: "Ubidreams._i18n",
    statics: {
      /*
        * Proxy for controller's getTranslation function
      */

      get: function(key, variables) {
        return scope.Ubidreams.i18n.impl.getTranslation(key, variables);
      },
      /*
        * Update a i18n object for a given control and setter
        * @param control: <enyo instance> the control to update
        * @param setter: <string> for what setter we update the i18n object
        * @param key: <string> new key to use
        * @param opts: <json> other options:
        * {
        *   translate: <boolean> DEFAULT(true) if true, execute the translate function afterward
        *   recursive_translate: See translate function
        *   params: See i18n object structure
        *   variables: See i18n object structure
        * }
      */

      setKey: function(control, setter, key, opts) {
        var setterStr, translation, _i, _len, _ref2, _ref3, _ref4, _ref5, _ref6;
        if ((_ref2 = control.i18n) == null) {
          control.i18n = [{}];
        }
        if (opts == null) {
          opts = {};
        }
        if ((_ref3 = opts.translate) == null) {
          opts.translate = true;
        }
        if ((_ref4 = opts.recursive_translate) == null) {
          opts.recursive_translate = false;
        }
        _ref5 = control.i18n;
        for (_i = 0, _len = _ref5.length; _i < _len; _i++) {
          translation = _ref5[_i];
          setterStr = (_ref6 = control.setter) != null ? _ref6 : "content";
          if (setterStr === setter) {
            translation.key = key;
            if (!(opts.variables == null)) {
              translation.variables = opts.variables;
            }
            if (!(opts.params == null)) {
              translation.params = opts.params;
            }
            if (opts.translate != null) {
              Ubidreams._i18n.translate(control, opts.recursive_translate);
            }
            return true;
          }
        }
        return false;
      },
      /*
        * Update variable array value for a given control and setter
        * @param control: <enyo instance> the control to update
        * @param setter: <string> for what setter we update the i18n object
        * @param opts: <json> other options:
        * {
        *   params: See i18n object structure
        *   variables: See i18n object structure
        *   recursive_translate: See translate function
        * }
      */

      setVariables: function(control, setter, opts) {
        var setterStr, translation, _i, _len, _ref2, _ref3, _ref4;
        if (opts == null) {
          opts = {};
        }
        if ((_ref2 = opts.recursive_translate) == null) {
          opts.recursive_translate = false;
        }
        _ref3 = control.i18n;
        for (_i = 0, _len = _ref3.length; _i < _len; _i++) {
          translation = _ref3[_i];
          setterStr = (_ref4 = control.setter) != null ? _ref4 : "content";
          if (setterStr === setter) {
            if (!(opts.variables == null)) {
              translation.variables = opts.variables;
            }
            if (!(opts.params == null)) {
              translation.params = opts.params;
            }
            Ubidreams._i18n.translate(control, opts.recursive_translate);
            return true;
          }
        }
        return false;
      },
      /*
        * Proxy for controller's _scanEnyoObject function
      */

      translate: function(control, recursive) {
        return scope.Ubidreams.i18n.impl._scanEnyoObject(control, recursive);
      }
    }
  });

}).call(this);
;
