cordova.define("com.ubidreams.DocumentInteraction.DocumentInteraction", function(require, exports, module) { module.exports = {
    /*
     * Open document method
     * @param <string> path
     * @param <string> {optional} mime
     * @param <boolean> {optional} shareOnly : default to false
     *                                         for Android change default itent ACTION_VIEW to ACTION_SEND
     *                                         see http://developer.android.com/guide/components/intents-filters.html
     * @param <json> {optional} opts : "onSuccess" and "onError" callbacks
     */
    openDocument: function(path, mime, shareOnly, opts) {
        var success = function () {
            console.log("DocumentInteraction :: openDocument :: success");
            if(opts.onSuccess != null){
                opts.onSuccess();
            }
        }
        var failure = function (error) {
            console.log("DocumentInteraction :: openDocument :: error " + error);
            if(opts.onError != null){
                opts.onError();
            }
        }

        // Check platform and encode path if necessary
        var encodedPath = null
        switch(cordova.platformId){
            case "android":
                encodedPath = this._encodeURI(path);
                break;
            default:
                encodedPath = path;
        }

        if (encodedPath != null)
            cordova.exec(success, failure, "DocumentInteraction", "openDocument", [encodedPath, mime, (shareOnly || false)]);
    },

    /*
     * Encode URI method
     * @param <string> uri
     */
    _encodeURI: function(uri){
        var newURL = "";
        var splittedURL = uri.split("/");

        for(var i=0; i<splittedURL.length; i++){
            if (i == 0){
                newURL += splittedURL[i]
            } else {
                newURL += "/";
                newURL += encodeURIComponent(splittedURL[i])
            }
        }

        return newURL;
    }
}
});
