// Generated by CoffeeScript 1.7.1
(function() {
  enyo.kind({
    name: "NumSync.Entry",
    classes: "entry",
    selected: false,
    published: {
      entry: void 0,
      button: 2,
      disabled: false
    },
    events: {
      onEntrySelect: "",
      onEntryTap: ""
    },
    handlers: {
      ontap: "_onEntryTap",
      ondown: "_enableActive",
      onup: "_disableActive",
      ondragstart: "_disableActive"
    },
    statics: {
      actions: {
        NONE: 0,
        SELECTION: 1,
        INFORMATION: 2,
        LIGHT: 3
      }
    },
    components: [
      {
        classes: "entry-icon",
        components: [
          {
            kind: "Image",
            name: "icon"
          }
        ]
      }, {
        classes: "entry-info",
        components: [
          {
            tag: "p",
            name: "entryName"
          }, {
            tag: "p",
            name: "info",
            allowHtml: true
          }
        ]
      }, {
        name: "action",
        classes: "entry-action",
        ontap: "_onDetailsTap",
        components: [
          {
            tag: "i",
            name: "btnInfo",
            classes: "fa fa-pencil",
            showing: false
          }, {
            kind: "onyx.Checkbox",
            name: "btnSelect",
            showing: false,
            onActivate: "_onCheckboxActivated"
          }
        ]
      }
    ],
    create: function() {
      this.inherited(arguments);
      this.entryChanged();
      return this.buttonChanged();
    },

    /*
      * Return true if entry is selected (in selection mode)
     */
    getSelected: function() {
      return this.$.btnSelect.getChecked();
    },

    /*
      * If true, set the entry selected (in selection mode)
     */
    setSelected: function(flag) {
      return this.$.btnSelect.setChecked(flag);
    },

    /*
      * Apply active state (blue background)
     */
    applyActiveState: function(flag) {
      if (flag === true) {
        return this.applyStyle("background", "rgba(160, 213, 255, 0.32) !important");
      } else {
        return this.applyStyle("background", null);
      }
    },
    entryChanged: function() {
      this._updateIcon();
      this._updateEntryName();
      return this._updateInfo();
    },
    buttonChanged: function() {
      this.$.btnInfo.setShowing(this.button === NumSync.Entry.actions.INFORMATION);
      this.$.btnSelect.setShowing(this.button === NumSync.Entry.actions.SELECTION);
      if (this.$.btnSelect.getShowing()) {
        this.$.btnSelect.setChecked(false);
      }
      this.$.action.addRemoveClass("information", this.button === NumSync.Entry.actions.INFORMATION);
      this.$.action.addRemoveClass("selection", this.button === NumSync.Entry.actions.SELECTION);
      if (this.button === NumSync.Entry.actions.NONE || this.button === NumSync.Entry.actions.SELECTION) {
        return this.disabled = true;
      } else {
        return this.disabled = false;
      }
    },

    /*
      * Update icon
     */
    _updateIcon: function() {
      var icon, type;
      if (this.entry != null) {
        icon = "Resources/icons/";
        type = this.entry.type;
        switch (type) {
          case NumSync.models.Entry.types.FOLDER:
            if (this.entry._share_ == null) {
              icon += "basic_folder.png";
            } else {
              icon += "my_shared_folder.png";
            }
            break;
          case NumSync.models.Entry.types.CLOUD_FOLDER:
            if (this.entry.myBox === true) {
              icon += "my_box.png";
            } else {
              if (this.entry._share_ == null) {
                icon += "basic_folder.png";
              } else {
                icon += "my_shared_folder.png";
              }
            }
            break;
          case NumSync.models.Entry.types.SHARED_FOLDER:
            icon += "shared_folder.png";
            break;
          case NumSync.models.Entry.types.FILE:
            icon += "files/" + this._checkExtensionForIcon();
            break;
          default:
            throw "Entry :: updateType :: " + type + " isn't a correct type.";
        }
      }
      return this.$.icon.setSrc(icon || "");
    },

    /*
      * Check extension for icon
      * @return <string> file path for icon
     */
    _checkExtensionForIcon: function() {
      return NumSync.config.SUPPORTED_EXTENSIONS[this.entry.path.split(".").pop().toLowerCase()] || "aut_0.png";
    },

    /*
      * Update entry name
     */
    _updateEntryName: function() {
      var _ref;
      switch ((_ref = this.entry) != null ? _ref.type : void 0) {
        case NumSync.models.Entry.types.FOLDER:
        case NumSync.models.Entry.types.CLOUD_FOLDER:
        case NumSync.models.Entry.types.SHARED_FOLDER:
        case NumSync.models.Entry.types.FILE:
          return this.$.entryName.setContent(this.entry.getEntryName());
        default:
          return this.$.entryName.setContent(">No name<");
      }
    },

    /*
      * Update details
     */
    _updateInfo: function() {
      var current, key, modified, value;
      if (this.entry != null) {
        switch (this.entry.type) {
          case NumSync.models.Entry.types.CLOUD_FOLDER:
            if (this.entry._share_ != null) {
              key = "shared_with";
              value = this.entry._share_.sharesByAccount.length;
            } else {
              this.$.entryName.addClass("no-date");
            }
            break;
          case NumSync.models.Entry.types.FOLDER:
          case NumSync.models.Entry.types.FILE:
            if (this.entry._share_ != null) {
              key = "shared_with";
              value = this.entry._share_.sharesByAccount.length;
            } else {
              modified = moment(this.entry.lastModified);
              current = moment();
              if (current.diff(modified, 'days') <= 1) {
                key = "modified";
                value = modified.fromNow();
              } else {
                key = "modified_on";
                value = modified.format("ll");
              }
            }
            break;
          case NumSync.models.Entry.types.SHARED_FOLDER:
            key = "shared_by";
            value = this.entry.owner.login;
            break;
          default:
            throw "Entry :: updateInfo :: " + this.entry.type + " isn't a correct type.";
        }
        return Ubidreams._i18n.setKey(this.$.info, "content", key, {
          variables: [value]
        });
      } else {
        return this.$.info.setContent("");
      }
    },
    _onDetailsTap: function() {
      if (this.button === NumSync.Entry.actions.INFORMATION) {
        enyo.Signals.send("showModal", {
          modal: {
            className: "NumSync.EntryDetails",
            config: {
              entry: this.entry
            }
          }
        });
        return true;
      }
    },
    _onEntryTap: function(inSender, inEvent) {
      if (this.disabled || (this.entry.type === NumSync.models.Entry.types.FILE && this.button === NumSync.Entry.actions.LIGHT)) {
        return;
      }
      return this.doEntryTap({});
    },
    _enableActive: function(inSender, inEvent) {
      if (inSender.name !== "action") {
        return this.applyActiveState(true);
      }
    },
    _disableActive: function(inSender, inEvent) {
      return this.applyActiveState(false);
    },
    _onCheckboxActivated: function(inSender, inEvent) {
      return this.doEntrySelect({
        selected: inSender.checked
      });
    }
  });

}).call(this);
